@@ -64,10 +64,12 @@ typedef struct xfs_trans {
 	unsigned int	t_log_res;		/* amt of log space resvd */
 	unsigned int	t_log_count;		/* count for perm log res */
 	unsigned int	t_blk_res;		/* # of blocks resvd */
+	unsigned int	t_blk_res_used;		/* # of resvd blocks used */
+	unsigned int	t_rtx_res;		/* # of rt extents resvd */
+	unsigned int	t_rtx_res_used;		/* # of resvd rt extents used */
+	unsigned int	t_flags;		/* misc flags */
 	xfs_fsblock_t	t_firstblock;		/* first block allocated */
 	struct xfs_mount *t_mountp;		/* ptr to fs mount struct */
-	unsigned int	t_blk_res_used;		/* # of resvd blocks used */
-	unsigned int	t_flags;		/* misc flags */
 	long		t_icount_delta;		/* superblock icount change */
 	long		t_ifree_delta;		/* superblock ifree change */
 	long		t_fdblocks_delta;	/* superblock fdblocks chg */
@@ -91,7 +93,7 @@ void	libxfs_trans_cancel(struct xfs_trans *);
 /* cancel dfops associated with a transaction */
 void xfs_defer_cancel(struct xfs_trans *);
 
-struct xfs_buf *libxfs_trans_getsb(struct xfs_trans *, struct xfs_mount *);
+struct xfs_buf *libxfs_trans_getsb(struct xfs_trans *);
 
 void	libxfs_trans_ijoin(struct xfs_trans *, struct xfs_inode *, uint);
 void	libxfs_trans_log_inode (struct xfs_trans *, struct xfs_inode *,
@@ -146,4 +148,29 @@ libxfs_trans_read_buf(
 	return libxfs_trans_read_buf_map(mp, tp, btp, &map, 1, flags, bpp, ops);
 }
 
+#define xfs_log_item_in_current_chkpt(lip)	(false)
+#define xfs_trans_item_relog(lip, tp)		(NULL)
+
+/* Contorted mess to make gcc shut up about unused vars. */
+#define xlog_grant_push_threshold(log, need)    \
+		((log) == (log) ? NULLCOMMITLSN : NULLCOMMITLSN)
+
+/* from xfs_log.h */
+/*
+ * By comparing each component, we don't have to worry about extra
+ * endian issues in treating two 32 bit numbers as one 64 bit number
+ */
+static inline xfs_lsn_t _lsn_cmp(xfs_lsn_t lsn1, xfs_lsn_t lsn2)
+{
+	if (CYCLE_LSN(lsn1) != CYCLE_LSN(lsn2))
+		return (CYCLE_LSN(lsn1)<CYCLE_LSN(lsn2))? -999 : 999;
+
+	if (BLOCK_LSN(lsn1) != BLOCK_LSN(lsn2))
+		return (BLOCK_LSN(lsn1)<BLOCK_LSN(lsn2))? -999 : 999;
+
+	return 0;
+}
+
+#define XFS_LSN_CMP(a, b)			_lsn_cmp(a, b)
+
 #endif	/* __XFS_TRANS_H__ */
