@@ -205,19 +205,19 @@ valuecheck(
  */
 static int
 process_shortform_attr(
-	struct xfs_mount *mp,
-	xfs_ino_t	ino,
-	xfs_dinode_t	*dip,
-	int		*repair)
+	struct xfs_mount		*mp,
+	xfs_ino_t			ino,
+	xfs_dinode_t			*dip,
+	int				*repair)
 {
-	xfs_attr_shortform_t	*asf;
-	xfs_attr_sf_entry_t	*currententry, *nextentry, *tempentry;
-	int			i, junkit;
-	int			currentsize, remainingspace;
+	struct xfs_attr_shortform	*asf;
+	struct xfs_attr_sf_entry	*currententry, *nextentry, *tempentry;
+	int				i, junkit;
+	int				currentsize, remainingspace;
 
 	*repair = 0;
 
-	asf = (xfs_attr_shortform_t *) XFS_DFORK_APTR(dip);
+	asf = (struct xfs_attr_shortform *) XFS_DFORK_APTR(dip);
 
 	/* Assumption: hdr.totsize is less than a leaf block and was checked
 	 * by lclinode for valid sizes. Check the count though.
@@ -250,7 +250,7 @@ process_shortform_attr(
 		junkit = 0;
 
 		/* don't go off the end if the hdr.count was off */
-		if ((currentsize + (sizeof(xfs_attr_sf_entry_t) - 1)) >
+		if ((currentsize + (sizeof(struct xfs_attr_sf_entry) - 1)) >
 						be16_to_cpu(asf->hdr.totsize))
 			break; /* get out and reset count and totSize */
 
@@ -314,7 +314,7 @@ process_shortform_attr(
 					currententry->valuelen);
 
 		remainingspace = remainingspace -
-					XFS_ATTR_SF_ENTSIZE(currententry);
+					xfs_attr_sf_entsize(currententry);
 
 		if (junkit) {
 			if (!no_modify) {
@@ -322,9 +322,9 @@ process_shortform_attr(
 				do_warn(
 	_("removing attribute entry %d for inode %" PRIu64 "\n"),
 					i, ino);
-				tempentry = (xfs_attr_sf_entry_t *)
+				tempentry = (struct xfs_attr_sf_entry *)
 					((intptr_t) currententry +
-					 XFS_ATTR_SF_ENTSIZE(currententry));
+					 xfs_attr_sf_entsize(currententry));
 				memmove(currententry,tempentry,remainingspace);
 				asf->hdr.count -= 1;
 				i--; /* no worries, it will wrap back to 0 */
@@ -338,9 +338,9 @@ process_shortform_attr(
 		}
 
 		/* Let's get ready for the next entry... */
-		nextentry = (xfs_attr_sf_entry_t *)((intptr_t) nextentry +
-			 		XFS_ATTR_SF_ENTSIZE(currententry));
-		currentsize = currentsize + XFS_ATTR_SF_ENTSIZE(currententry);
+		nextentry = (struct xfs_attr_sf_entry *)((intptr_t) nextentry +
+			 		xfs_attr_sf_entsize(currententry));
+		currentsize = currentsize + xfs_attr_sf_entsize(currententry);
 
 	} /* end the loop */
 
@@ -424,9 +424,9 @@ rmtval_get(xfs_mount_t *mp, xfs_ino_t ino, blkmap_t *blkmap,
 			break;
 		}
 
-		ASSERT(mp->m_sb.sb_blocksize == bp->b_bcount);
+		ASSERT(mp->m_sb.sb_blocksize == BBTOB(bp->b_length));
 
-		length = min(bp->b_bcount - hdrsize, valuelen - amountdone);
+		length = min(BBTOB(bp->b_length) - hdrsize, valuelen - amountdone);
 		memmove(value, bp->b_addr + hdrsize, length);
 		amountdone += length;
 		value += length;
@@ -1212,9 +1212,9 @@ process_attributes(
 	int		err;
 	__u8		aformat = dip->di_aformat;
 #ifdef DEBUG
-	xfs_attr_shortform_t *asf;
+	struct xfs_attr_shortform *asf;
 
-	asf = (xfs_attr_shortform_t *) XFS_DFORK_APTR(dip);
+	asf = (struct xfs_attr_shortform *) XFS_DFORK_APTR(dip);
 #endif
 
 	if (aformat == XFS_DINODE_FMT_LOCAL) {
