@@ -92,7 +92,7 @@ _("would have cleared inode %" PRIu64 " attributes\n"), ino_num);
 	 */
 
 	if (!no_modify) {
-		xfs_attr_shortform_t *asf = (xfs_attr_shortform_t *)
+		struct xfs_attr_shortform *asf = (struct xfs_attr_shortform *)
 				XFS_DFORK_APTR(dino);
 		asf->hdr.totsize = cpu_to_be16(sizeof(xfs_attr_sf_hdr_t));
 		asf->hdr.count = 0;
@@ -176,6 +176,103 @@ verify_dfsbno_range(
 
 	return XR_DFSBNORANGE_VALID;
 }
+
+static int
+process_rt_rec_dups(
+	struct xfs_mount	*mp,
+	xfs_ino_t		ino,
+	struct xfs_bmbt_irec	*irec)
+{
+	xfs_fsblock_t		b;
+	xfs_rtblock_t		ext;
+
+	for (b = rounddown(irec->br_startblock, mp->m_sb.sb_rextsize);
+	     b < irec->br_startblock + irec->br_blockcount;
+	     b += mp->m_sb.sb_rextsize) {
+		ext = (xfs_rtblock_t) b / mp->m_sb.sb_rextsize;
+		if (search_rt_dup_extent(mp, ext))  {
+			do_warn(
+_("data fork in rt ino %" PRIu64 " claims dup rt extent,"
+"off - %" PRIu64 ", start - %" PRIu64 ", count %" PRIu64 "\n"),
+				ino,
+				irec->br_startoff,
+				irec->br_startblock,
+				irec->br_blockcount);
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+static int
+process_rt_rec_state(
+	struct xfs_mount	*mp,
+	xfs_ino_t		ino,
+	struct xfs_bmbt_irec	*irec)
+{
+	xfs_fsblock_t		b = irec->br_startblock;
+	xfs_rtblock_t		ext;
+	int			state;
+
+	do {
+		ext = (xfs_rtblock_t)b / mp->m_sb.sb_rextsize;
+		state = get_rtbmap(ext);
+
+		if ((b % mp->m_sb.sb_rextsize) != 0) {
+			/*
+			 * We are midway through a partially written extent.
+			 * If we don't find the state that gets set in the
+			 * other clause of this loop body, then we have a
+			 * partially *mapped* rt extent and should complain.
+			 */
+			if (state != XR_E_INUSE)
+				do_error(
+_("data fork in rt inode %" PRIu64 " found invalid rt extent %"PRIu64" state %d at rt block %"PRIu64"\n"),
+					ino, ext, state, b);
+			b = roundup(b, mp->m_sb.sb_rextsize);
+			continue;
+		}
+
+		/*
+		 * This is the start of an rt extent.  Set the extent state if
+		 * nobody else has claimed the extent, or complain if there are
+		 * conflicting states.
+		 */
+		switch (state)  {
+		case XR_E_FREE:
+		case XR_E_UNKNOWN:
+			set_rtbmap(ext, XR_E_INUSE);
+			break;
+		case XR_E_BAD_STATE:
+			do_error(
+_("bad state in rt extent map %" PRIu64 "\n"),
+				ext);
+		case XR_E_FS_MAP:
+		case XR_E_INO:
+		case XR_E_INUSE_FS:
+			do_error(
+_("data fork in rt inode %" PRIu64 " found rt metadata extent %" PRIu64 " in rt bmap\n"),
+				ino, ext);
+		case XR_E_INUSE:
+		case XR_E_MULT:
+			set_rtbmap(ext, XR_E_MULT);
+			do_warn(
+_("data fork in rt inode %" PRIu64 " claims used rt extent %" PRIu64 "\n"),
+				ino, b);
+			return 1;
+		case XR_E_FREE1:
+		default:
+			do_error(
+_("illegal state %d in rt extent %" PRIu64 "\n"),
+				state, ext);
+		}
+		b += mp->m_sb.sb_rextsize;
+	} while (b < irec->br_startblock + irec->br_blockcount);
+
+	return 0;
+}
+
 static int
 process_rt_rec(
 	struct xfs_mount	*mp,
@@ -184,10 +281,8 @@ process_rt_rec(
 	xfs_rfsblock_t		*tot,
 	int			check_dups)
 {
-	xfs_fsblock_t		b, lastb;
-	xfs_rtblock_t		ext;
-	int			state;
-	int			pwe;		/* partially-written extent */
+	xfs_fsblock_t		lastb;
+	int			bad;
 
 	/*
 	 * check numeric validity of the extent
@@ -221,63 +316,12 @@ _("inode %" PRIu64 " - bad rt extent overflows - start %" PRIu64 ", "
 		return 1;
 	}
 
-	/*
-	 * set the appropriate number of extents
-	 * this iterates block by block, this can be optimised using extents
-	 */
-	for (b = irec->br_startblock; b < irec->br_startblock +
-			irec->br_blockcount; b += mp->m_sb.sb_rextsize)  {
-		ext = (xfs_rtblock_t) b / mp->m_sb.sb_rextsize;
-		pwe = irec->br_state == XFS_EXT_UNWRITTEN &&
-				(b % mp->m_sb.sb_rextsize != 0);
-
-		if (check_dups == 1)  {
-			if (search_rt_dup_extent(mp, ext) && !pwe)  {
-				do_warn(
-_("data fork in rt ino %" PRIu64 " claims dup rt extent,"
-  "off - %" PRIu64 ", start - %" PRIu64 ", count %" PRIu64 "\n"),
-					ino,
-					irec->br_startoff,
-					irec->br_startblock,
-					irec->br_blockcount);
-				return 1;
-			}
-			continue;
-		}
-
-		state = get_rtbmap(ext);
-		switch (state)  {
-		case XR_E_FREE:
-		case XR_E_UNKNOWN:
-			set_rtbmap(ext, XR_E_INUSE);
-			break;
-		case XR_E_BAD_STATE:
-			do_error(
-_("bad state in rt block map %" PRIu64 "\n"),
-				ext);
-		case XR_E_FS_MAP:
-		case XR_E_INO:
-		case XR_E_INUSE_FS:
-			do_error(
-_("data fork in rt inode %" PRIu64 " found metadata block %" PRIu64 " in rt bmap\n"),
-				ino, ext);
-		case XR_E_INUSE:
-			if (pwe)
-				break;
-			/* fall through */
-		case XR_E_MULT:
-			set_rtbmap(ext, XR_E_MULT);
-			do_warn(
-_("data fork in rt inode %" PRIu64 " claims used rt block %" PRIu64 "\n"),
-				ino, ext);
-			return 1;
-		case XR_E_FREE1:
-		default:
-			do_error(
-_("illegal state %d in rt block map %" PRIu64 "\n"),
-				state, b);
-		}
-	}
+	if (check_dups)
+		bad = process_rt_rec_dups(mp, ino, irec);
+	else
+		bad = process_rt_rec_state(mp, ino, irec);
+	if (bad)
+		return bad;
 
 	/*
 	 * bump up the block counter
@@ -927,14 +971,14 @@ process_exinode(
  */
 static int
 process_lclinode(
-	xfs_mount_t		*mp,
-	xfs_agnumber_t		agno,
-	xfs_agino_t		ino,
-	xfs_dinode_t		*dip,
-	int			whichfork)
+	xfs_mount_t			*mp,
+	xfs_agnumber_t			agno,
+	xfs_agino_t			ino,
+	xfs_dinode_t			*dip,
+	int				whichfork)
 {
-	xfs_attr_shortform_t	*asf;
-	xfs_ino_t		lino;
+	struct xfs_attr_shortform	*asf;
+	xfs_ino_t			lino;
 
 	lino = XFS_AGINO_TO_INO(mp, agno, ino);
 	if (whichfork == XFS_DATA_FORK && be64_to_cpu(dip->di_size) >
@@ -945,7 +989,7 @@ process_lclinode(
 			XFS_DFORK_DSIZE(dip, mp));
 		return(1);
 	} else if (whichfork == XFS_ATTR_FORK) {
-		asf = (xfs_attr_shortform_t *)XFS_DFORK_APTR(dip);
+		asf = (struct xfs_attr_shortform *)XFS_DFORK_APTR(dip);
 		if (be16_to_cpu(asf->hdr.totsize) > XFS_DFORK_ASIZE(dip, mp)) {
 			do_warn(
 	_("local inode %" PRIu64 " attr fork too large (size %d, max = %zu)\n"),
@@ -1348,17 +1392,6 @@ process_misc_ino_types(xfs_mount_t	*mp,
 			xfs_ino_t	lino,
 			int		type)
 {
-	/*
-	 * disallow mountpoint inodes until such time as the
-	 * kernel actually allows them to be created (will
-	 * probably require a superblock version rev, sigh).
-	 */
-	if (type == XR_INO_MOUNTPOINT)  {
-		do_warn(
-_("inode %" PRIu64 " has bad inode type (IFMNT)\n"), lino);
-		return(1);
-	}
-
 	/*
 	 * must also have a zero size
 	 */
@@ -1644,7 +1677,6 @@ _("directory inode %" PRIu64 " has bad size %" PRId64 "\n"),
 	case XR_INO_CHRDEV:	/* fall through to FIFO case ... */
 	case XR_INO_BLKDEV:	/* fall through to FIFO case ... */
 	case XR_INO_SOCK:	/* fall through to FIFO case ... */
-	case XR_INO_MOUNTPOINT:	/* fall through to FIFO case ... */
 	case XR_INO_FIFO:
 		if (process_misc_ino_types(mp, dino, lino, type))
 			return 1;
@@ -2123,10 +2155,17 @@ static void
 check_nsec(
 	const char		*name,
 	xfs_ino_t		lino,
-	struct xfs_timestamp	*t,
+	struct xfs_dinode	*dip,
+	xfs_timestamp_t		*ts,
 	int			*dirty)
 {
-	if (be32_to_cpu(t->t_nsec) < 1000000000)
+	struct xfs_legacy_timestamp *t;
+
+	if (xfs_dinode_has_bigtime(dip))
+		return;
+
+	t = (struct xfs_legacy_timestamp *)ts;
+	if (be32_to_cpu(t->t_nsec) < NSEC_PER_SEC)
 		return;
 
 	do_warn(
@@ -2547,6 +2586,27 @@ _("bad (negative) size %" PRId64 " on inode %" PRIu64 "\n"),
 			flags2 &= ~XFS_DIFLAG2_COWEXTSIZE;
 		}
 
+		if (xfs_dinode_has_bigtime(dino) &&
+		    !xfs_sb_version_hasbigtime(&mp->m_sb)) {
+			if (!uncertain) {
+				do_warn(
+	_("inode %" PRIu64 " is marked bigtime but file system does not support large timestamps\n"),
+					lino);
+			}
+			flags2 &= ~XFS_DIFLAG2_BIGTIME;
+
+			if (no_modify) {
+				do_warn(_("would zero timestamps.\n"));
+			} else {
+				do_warn(_("zeroing timestamps.\n"));
+				dino->di_atime = 0;
+				dino->di_mtime = 0;
+				dino->di_ctime = 0;
+				dino->di_crtime = 0;
+				*dirty = 1;
+			}
+		}
+
 		if (!verify_mode && flags2 != be64_to_cpu(dino->di_flags2)) {
 			if (!no_modify) {
 				do_warn(_("fixing bad flags2.\n"));
@@ -2674,11 +2734,11 @@ _("Bad CoW extent size %u on inode %" PRIu64 ", "),
 	}
 
 	/* nsec fields cannot be larger than 1 billion */
-	check_nsec("atime", lino, &dino->di_atime, dirty);
-	check_nsec("mtime", lino, &dino->di_mtime, dirty);
-	check_nsec("ctime", lino, &dino->di_ctime, dirty);
+	check_nsec("atime", lino, dino, &dino->di_atime, dirty);
+	check_nsec("mtime", lino, dino, &dino->di_mtime, dirty);
+	check_nsec("ctime", lino, dino, &dino->di_ctime, dirty);
 	if (dino->di_version >= 3)
-		check_nsec("crtime", lino, &dino->di_crtime, dirty);
+		check_nsec("crtime", lino, dino, &dino->di_crtime, dirty);
 
 	/*
 	 * general size/consistency checks:
