@@ -3,7 +3,7 @@
  * Copyright (c) 2000-2003,2005 Silicon Graphics, Inc.
  * All Rights Reserved.
  */
-#include "libfrog.h"
+#include "libfrog/util.h"
 #include "libxfs.h"
 #include "libxcmd.h"
 #include "libxlog.h"
@@ -12,6 +12,7 @@
 #include "protos.h"
 #include "err_protos.h"
 #include "xfs_multidisk.h"
+#include "libfrog/topology.h"
 
 #define BSIZE	(1024 * 1024)
 
@@ -57,8 +58,7 @@ copy_sb(xfs_sb_t *source, xfs_sb_t *dest)
 	 */
 	if (xfs_sb_version_hasdalign(source))
 		dest->sb_versionnum |= XFS_SB_VERSION_DALIGNBIT;
-	if (xfs_sb_version_hasextflgbit(source))
-		dest->sb_versionnum |= XFS_SB_VERSION_EXTFLGBIT;
+	dest->sb_versionnum |= XFS_SB_VERSION_EXTFLGBIT;
 
 	/*
 	 * these are all supposed to be zero or will get reset anyway
@@ -369,8 +369,7 @@ verify_sb(char *sb_buf, xfs_sb_t *sb, int is_primary_sb)
 		return(XR_BAD_VERSION);
 
 	/* does sb think mkfs really finished ? */
-
-	if (is_primary_sb && sb->sb_inprogress == 1)
+	if (is_primary_sb && sb->sb_inprogress)
 		return(XR_BAD_INPROGRESS);
 
 	/*
@@ -683,9 +682,7 @@ get_sb_geometry(fs_geometry_t *geo, xfs_sb_t *sbp)
 	if (xfs_sb_version_hasdalign(sbp))
 		geo->sb_salignbit = 1;
 
-	if (xfs_sb_version_hasextflgbit(sbp))
-		geo->sb_extflgbit = 1;
-
+	geo->sb_extflgbit = 1;
 	geo->sb_fully_zeroed = 1;
 }
 
